/*-
 * Copyright (C)2008..2021 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2008..2021 @BABOLO http://www.babolo.ru/\n"
#ident "@(#) All rights reserved.\n"
#ident "@(#) $Id: opts.c,v 1.9 2021/06/06 17:37:13 babolo Exp $\n"

#define BLIN_COMPAT  4
#define Bpars_COMPAT VMAJOR

#include <sys/types.h>
#include <sysexits.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <stdio.h>
#include <errno.h>
#include <babolo/BLINflag.h>
#include "../parser.h"

int
main(int argc, char **argv) {
    BLIN_flag flags = Bpars_NONU | Bpars_ESYM | Bpars_EPRM | Bpars_CMPT | Bpars_PREA;
#   define blin_internal_flags (flags & BLIN_MASK)
    const char  *flar = "a:b:f:vDR:@:::::::::::::::::=:";
    const char  *Tflag = NULL;
    babolo_opts *bos;
    babolo_parm *bp;
    const char  *ss;
    int          c;
    int          i;
    char        *s;

    if  (!!(s = getenv("INIT_FLAGS"))) {
        flags = strtoul(s, NULL, 0) | Bpars_PREA;
    }
    if  (!(bos = babolo_openopts(flags, 0))) {
        ifBLIN_QW0("openopts");
        exit(EX_SOFTWARE);
    }
    if  (!!babolo_setopts(bos, flags, argc, argv, flar)) {
        ifBLIN_QW0("setopts");
        exit(EX_SOFTWARE);
    }
    ifBLIN_QO7 babolo_dumpopts(bos);
    for (i = 0;; ++i) {
        switch ((c = babolo_getopts(bos))) {
        case '=': switch((Tflag = babolo_getoptsarg(bos))[0]) {
                  case '0': blin_ctl(BLIN_CTL_DUMP); exit(EX_OK);
                  case '1': babolo_optest(flar);     exit(EX_OK);
                  case '2': babolo_optext(bos);      exit(EX_OK);
                  case '3': babolo_dumpopts(bos);    exit(EX_OK);
                  default :                          break;
                  }
                ; break;
        case '@': for (c = 0; c < 17; ++c) {
                      ss = babolo_getoptsarg(bos);
                      if  (!!ss) {
                          printf("-@ [%d]=%s~\n", c, ss);
                      } else {
                          printf("-@ [%d]\n", c);
                  }   }
                ; break;
        case 0  :                                    break;
        case -1 : printf("ERR %d\n", errno);         break;
        case 'v':
            BLIN_VERBOSE(flags);
            bos->flags |= flags & BLIN_MASK;
            break;
        case 'R': ss = babolo_getoptsarg(bos);
                  if  (!!ss) {
                      s = strdup(ss);
                      bp = babolo_getparm(flags & Bpars_NOEN, &s, "", 0);
                      babolo_setparm(bos, (flags | Bpars_CMPT) & ~Bpars_NONU, bp, NULL);
                      ifBLIN_QO7 babolo_dumpopts(bos);
                      break;
                  }
        case 'f': ss = babolo_getoptsarg(bos);
                  if  (!!ss) flags = strtoul(ss, NULL, 0) | (flags & BLIN_MASK);
        default : ss = babolo_getoptsarg(bos);
                  if  (ss) {
                      printf(" %c=%s~\n", c, ss);
                  } else {
                      printf(" %c NULL\n", c);
        }         }
        if  (c == 0) break;
    }
    for (i = 0; (ss = babolo_getargs(bos)); ++i) {
        printf("%d=%s~\n", i, ss);
    }
    ifBLIN_QO7 babolo_dumpopts(bos);
    babolo_closeopts(bos);
    exit(0);
#   undef blin_internal_flags
}
